#property strict

#include <Indicator.mqh>
#include <Quotes_FXOPEN.mqh>

class BOTHPRICES : public INDICATOR
{
private:
  datetime StartDate; //  
  double MarkUp;      //  ( + )   
  bool ShowVol;       //  " " (HighAsk - LowBid)

  datetime CalcIndicator( const datetime FirstDate )
  {
    QUOTES_FXOPEN Quotes(Symbol(), FirstDate);

    Quotes.SetMarkUp(this.MarkUp);

    const int Amount = Quotes.GetAmount();

    for (int i = 0; i < Amount; i++)
    {
    // QUOTES    ,    MT4
      const int j = iBarShift(Symbol(), Period(), Quotes[i].time - PeriodSeconds(Quotes.GetTimeFrame()), TRUE);

      if (j != -1)
      {
        if (this.ShowVol)
        {
          this.Buffers[0].Buffer[j] = Quotes[i].ask.max * _Point;
          this.Buffers[1].Buffer[j] = Quotes[i].bid.min * _Point;
        }

        this.Buffers[2].Buffer[j] = Quotes[i].bid.max * _Point;
        this.Buffers[3].Buffer[j] = Quotes[i].ask.min * _Point;
      }
    }

    //  
    ArrayCopy(this.Buffers[4].Buffer, this.Buffers[2].Buffer);
    ArrayCopy(this.Buffers[5].Buffer, this.Buffers[3].Buffer);

    if (!this.ShowVol)
    {
      this.Buffers[0].SetEmptyValue();
      this.Buffers[1].SetEmptyValue();
    }

    return(Quotes.GetLastTime() - PeriodSeconds(Quotes.GetTimeFrame()));
  }

// -    
  virtual void Event( const int id, const long &lparam, const double &dparam, const string &sparam )
  {
    if (id == CHARTEVENT_CHART_CHANGE)
    {
      long Scale;

      if (ChartGetInteger(0, CHART_SCALE, 0, Scale))
      {
        const int Width = 9 / (1 << (5 - (int)Scale));

        this.Buffers[2].SetWidth(Width, FALSE);
        this.Buffers[3].SetWidth(Width);
      }
    }

    return;
  }

  // 
  static void ChangeBufferColor( BUFFER* &Buffer, const int PartColor = 0xFF, const bool Redraw = TRUE )
  {
    static int Add = 40;

    color Color = Buffer.GetColor();
    const uchar Byte = (uchar)((Color & PartColor) / (PartColor / 0xFF));

    if (((Add < 0) && (Byte < -Add)) || ((Add > 0) && (Byte > 0xFF - Add)))
      Add = - Add;

    Color = (color)((Color & (0xFFFFFF - PartColor)) + (Byte + Add) * PartColor / 0xFF);

    Buffer.SetColor(Color, Redraw);

    return;
  }

// -  
  virtual void EventTimer( void )
  {
    if (this.Buffers[4].GetStyle() != STYLE_DASH)
    {
      this.Buffers[4].SetStyle(STYLE_DASH, FALSE);
      this.Buffers[5].SetStyle(STYLE_DASHDOT, FALSE);
    }
    else
    {
      this.Buffers[4].SetStyle(STYLE_DOT, FALSE);
      this.Buffers[5].SetStyle(STYLE_DASHDOTDOT, FALSE);
    }

//    ChangeBufferColor(this.Buffers[2], 0xFF00, FALSE);
    ChangeBufferColor(this.Buffers[3]);

    return;
  }

public:
  void Init( const datetime tStartDate = NULL, const double dMarkUp = 0, const bool bShowVol = FALSE )
  {
    this.StartDate = tStartDate;
    this.MarkUp = dMarkUp;
    this.ShowVol = bShowVol;

    return;
  }

  BOTHPRICES( const datetime tStartDate = NULL, const double dMarkUp = 0, const bool bShowVol = FALSE,
              const bool AddEvent = FALSE  ) : INDICATOR(AddEvent), StartDate(tStartDate), MarkUp(dMarkUp), ShowVol(bShowVol)
  {
    const string Names[] = {"HighAsk", "LowBid", "HighBid", "LowAsk", "HighBid_Line", "LowAsk_Line"};
    const int Types[] = {DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_HISTOGRAM, DRAW_LINE, DRAW_LINE};
    const color Colors[] = {clrYellow, clrYellow, clrBlue, clrRed, clrDeepSkyBlue, clrRed};

    const int Size = ArraySize(Names);

    ArrayResize(this.Buffers, Size);

    for (int i = 0; i < Size; i++)
      this.Buffers[i] = new BUFFER(Types[i], EMPTY, EMPTY, Colors[i], Names[i]);

    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
  }

  virtual int Calculate( const int rates_total,
                         const int prev_calculated,
                         const datetime &time[],
                         const double &open[],
                         const double &high[],
                         const double &low[],
                         const double &close[],
                         const long &tick_volume[],
                         const long &volume[],
                         const int &spread[] )
  {
    static datetime LastTime = NULL;

    const datetime FirstTime = time[this.CorrectIndex(rates_total - prev_calculated, rates_total)];

    if (LastTime > FirstTime)
      LastTime = FirstTime;

    if (LastTime < this.StartDate)
      LastTime = this.StartDate;

    // MT4    ,    
    if (prev_calculated == 0)
      this.SetEmpty();

    LastTime = CalcIndicator(LastTime);

    return(rates_total);
  }
};