#property strict

#include <EventBase.mqh>

class BUFFER : public EVENTBASE
{
private:
  static int BuffersTotal;

  const int Type;
  ENUM_LINE_STYLE Style;
  int Width;
  color  Color;

  string Name;

  void _SetIndexStyle( const bool Redraw = FALSE ) const
  {
    if (this.MT4IndicatorBuffer)
    {
      SetIndexStyle(this.Index, this.Type, this.Style, this.Width, this.Color);

      if (Redraw)
        ChartRedraw();
    }
    return;
  }

public:
  double Buffer[];

  const int Index;
  const bool MT4IndicatorBuffer;

  void SetName( const string sName = "" )
  {
    if (this.Name != sName)
    {
      this.Name = sName;

      if (this.MT4IndicatorBuffer)
        SetIndexLabel(this.Index, this.Name);
    }

    return;
  }

  void SetWidth( const int iWidth = EMPTY, const bool Redraw = TRUE )
  {
    if (this.Width != iWidth)
    {
      this.Width = iWidth;

      this._SetIndexStyle(Redraw);
    }

    return;
  }

  void SetStyle( const ENUM_LINE_STYLE iStyle = EMPTY, const bool Redraw = TRUE )
  {
    if (this.Style != iStyle)
    {
      this.Style = iStyle;

      this._SetIndexStyle(Redraw);
    }

    return;
  }

  void SetColor( const color cColor = clrNONE, const bool Redraw = TRUE )
  {
    if (this.Color != cColor)
    {
      this.Color = cColor;

      this._SetIndexStyle(Redraw);
    }

    return;
  }

  void SetEmptyValue( void )
  {
    const int Size = ArraySize(this.Buffer);

    if (Size > 0)
      ArrayFill(this.Buffer, 0, Size, EMPTY_VALUE);

    return;
  }

  string GetName( void ) const
  {
    return(this.Name);
  }

  int GetWidth( void ) const
  {
    return(this.Width);
  }

  ENUM_LINE_STYLE GetStyle( void ) const
  {
    return(this.Style);
  }

  color GetColor( void ) const
  {
    return(this.Color);
  }

  // MT4- 
  BUFFER( const int iType, const ENUM_LINE_STYLE eStyle = EMPTY,
          const int iWidth = EMPTY, const color cColor = clrNONE, const string sName = "", const bool AddEvent = FALSE ) :
          EVENTBASE(AddEvent), Type(iType), Style(eStyle), Width(iWidth), Color(cColor),
          Name(""), Index(BUFFER::BuffersTotal), MT4IndicatorBuffer(TRUE)
  {
    SetIndexBuffer(this.Index, this.Buffer);

    this._SetIndexStyle();

    this.SetName(sName);

    this.BuffersTotal++;
  }

  //  MT4- 
  BUFFER( const string sName = "", const bool AddEvent = FALSE ) : EVENTBASE(AddEvent),
          Type(EMPTY), Style(EMPTY), Width(EMPTY), Color(clrNONE), Name(sName), Index(EMPTY), MT4IndicatorBuffer(FALSE)
  {
  }

  double operator []( const int Pos ) const
  {
    return(this.Buffer[Pos]);
  }

  static int GetMT4IndicatorBuffersTotal( void )
  {
    return(BUFFER::BuffersTotal);
  }
};

int BUFFER::BuffersTotal = 0;