#property strict

#include <Quote.mqh>

#define DATETIME_MAX (D'3000.12.31 23:59:59')

class QUOTES
{
private:
  //  ()  SourceQuotes   (,   BeginPos)   PosTime
  //   :   
  static int SearchQuoteByTime( const QUOTES &SourceQuotes, const datetime PosTime, int BeginPos = 0 );

protected:
  QUOTE Quotes[];

  string Symbolname;
  int TimeFrame;

  //   ,    NULL
  void SetTimeFrame( const int iTimeFrame = NULL );

  static string CheckSymbolName(  string Symb = NULL );

  //  ()  SourceQuotes   (,   BeginPos)   PosTime
  //   :   
  template <typename T>
  static int SearchQuoteByTime( const T &SourceQuotes[], const datetime PosTime, int BeginPos = 0 );

  // FlagCopy  == FALSE -     
  template <typename T>
  static bool MyArrayResize( T &Array[], const int Amount, const string FuncName,
                             const bool FlagCopy = TRUE, const int ReserveSize = 0 );
public:

  QUOTES( const string sSymbolName, const int iTimeFrame = NULL ) : Symbolname(CheckSymbolName(sSymbolName)), TimeFrame(Period())
  {
    SetTimeFrame(iTimeFrame);
  }

  //      ( ),    AnotherQuotes
  QUOTES( const QUOTES &AnotherQuotes,  const string sSymbolName ) : Symbolname(CheckSymbolName(sSymbolName)), TimeFrame(Period())
  {
    SetTimeFrame(AnotherQuotes.GetTimeFrame());
  }

  string GetSymbolName( void ) const;
  int GetAmount( void ) const;
  int GetTimeFrame( void ) const;

  //      (double),    (int).     !
  template <typename T>
  void SetMarkUp( const T MarkUp );

  const QUOTE operator []( const int Pos ) const;

  //    ,  ()   PosTime
  const QUOTE operator []( const datetime PosTime ) const;

  datetime GetFirstTime( void ) const;
  datetime GetLastTime( void ) const;

  //          -     (AnotherQuotes)
  virtual bool Init( const datetime BeginTime = NULL, const datetime EndTime = NULL, const int iTimeFrame = NULL );
  virtual bool Init( const QUOTES &AnotherQuotes );

  //    /   Times[] (QuotesTimes) -     -.
  void AddTimePoints( const datetime &Times[] );
//  void AddTimePoints( const QUOTES* QuotesTimes );

  //     [BeginTime; EndTime]
  void FilterByInterval( const datetime BeginTime, datetime EndTime = NULL );

  //      
  static void FilterByInterval( QUOTES* &ArrayQuotes[] );

  //        
  static void Synchronize( QUOTES* &ArrayQuotes[] );
};

//  ()  SourceQuotes   (,   BeginPos)   PosTime
//   :   
static int QUOTES::SearchQuoteByTime( const QUOTES &SourceQuotes, const datetime PosTime, int BeginPos = 0 )
{
  int Res;
  const int Amount = SourceQuotes.GetAmount();

  if (BeginPos < 0)
    BeginPos = 0;

  for (Res = BeginPos + 1; (Res < Amount) && (SourceQuotes[Res].time <= PosTime); Res++) //    
    ;

  Res--;

  return(Res);
}

template <typename T>
static int QUOTES::SearchQuoteByTime( const T &SourceQuotes[], const datetime PosTime, int BeginPos = 0 )
{
  int Res;
  const int Amount = ArraySize(SourceQuotes);

  if (BeginPos < 0)
    BeginPos = 0;

  for (Res = BeginPos + 1; (Res < Amount) && (SourceQuotes[Res].time <= PosTime); Res++) //    
    ;

  Res--;

  return(Res);
}

//   ,    NULL
void QUOTES::SetTimeFrame( const int iTimeFrame = NULL )
{
  if (iTimeFrame != NULL)
    this.TimeFrame = iTimeFrame;

  return;
}

static string QUOTES::CheckSymbolName(  string Symb = NULL )
{
  if (Symb == NULL)
    Symb = Symbol();

  return(Symb);
}

// FlagCopy  == FALSE -     
template <typename T>
static bool QUOTES::MyArrayResize( T &Array[], const int Amount, const string FuncName,
                                   const bool FlagCopy = TRUE, const int ReserveSize = 0 )
{
  if (!FlagCopy && (Amount + ReserveSize > ArraySize(Array)))
    ArrayFree(Array);

  const bool Res = (ArrayResize(Array, Amount, ReserveSize) == Amount);

  if (!Res) //        ( )
    Alert(FuncName + ": "+ "ArrayResize(" + typename(T) + ", " + (string)Amount + ") - error " + (string)GetLastError());

  return(Res);
}

//      
static void QUOTES::FilterByInterval( QUOTES* &ArrayQuotes[] )
{
  const int Amount = ArraySize(ArrayQuotes);

  datetime FirstTime = 0;
  datetime LastTime = DATETIME_MAX;

  for (int i = 0; i < Amount; i++)
    if (ArrayQuotes[i] != NULL)
    {
      if (ArrayQuotes[i].GetFirstTime() > FirstTime)
        FirstTime = ArrayQuotes[i].GetFirstTime();

      if (ArrayQuotes[i].GetLastTime() < LastTime)
        LastTime = ArrayQuotes[i].GetLastTime();
    }

  for (int i = 0; i < Amount; i++)
    if (ArrayQuotes[i] != NULL)
      ArrayQuotes[i].FilterByInterval(FirstTime, LastTime);

  return;
}

string QUOTES::GetSymbolName( void ) const
{
  return(this.Symbolname);
}

int QUOTES::GetAmount( void ) const
{
  return(ArraySize(this.Quotes));
}

int QUOTES::GetTimeFrame( void ) const
{
  return(this.TimeFrame);
}

//      (double),    (int).     !
template <typename T>
void QUOTES::SetMarkUp( const T MarkUp )
{
  const int Amount = this.GetAmount();

  for (int i = 0; i < Amount; i++)
    this.Quotes[i].SetMarkUp(MarkUp);

  return;
}

const QUOTE QUOTES::operator []( const int Pos ) const
{
  if ((Pos >= 0) && (Pos < this.GetAmount()))
    return(this.Quotes[Pos]);

  const QUOTE Res;

  return(Res);
}

//    ,  ()   PosTime
const QUOTE QUOTES::operator []( const datetime PosTime ) const
{
  return(this.Quotes[SearchQuoteByTime(this, PosTime)]);
}

datetime QUOTES::GetFirstTime( void ) const
{
  return(this[0].time);
}

datetime QUOTES::GetLastTime( void ) const
{
  return(this[this.GetAmount() - 1].time);
}

//    /   Times[] (QuotesTimes) -     -.
void QUOTES::AddTimePoints( const datetime &Times[] )
//  void QUOTES::AddTimePoints( const QUOTES* QuotesTimes )
{
//    if ((QuotesTimes == NULL) || (GetPointer(this) == QuotesTimes))
//      return;

//    const int Amount = QuotesTimes.GetAmount();
  const int Amount = ArraySize(Times);

  if ((Amount <= 0) || (this.GetAmount() <= 0))
    return;

  int NewAmount = 0;
  int Pos = 0, PrevPos = Pos;

  for (int j = 0; j < Amount; j++)
  {
    Pos = SearchQuoteByTime(this, Times[j], Pos);
//      Pos = SearchQuoteByTime(this, QuotesTimes[j].time, Pos);

    if (this[Pos].time < Times[j])
//      if (this[Pos].time < QuotesTimes[j].time)
    {
      NewAmount += Pos - PrevPos + 2;

      PrevPos = Pos + 1;
    }
  }

  //  PrevPos == 0    ,  
  if (PrevPos == 0)
    return;

  NewAmount += this.GetAmount() - PrevPos;

  QUOTE TmpQuotes[];

  //        (  )
  if (!this.MyArrayResize(TmpQuotes, NewAmount, __FUNCTION__))
    return;

  NewAmount = 0;
  Pos = 0;
  PrevPos = Pos;

  // 
  for (int j = 0; j < Amount; j++)
  {
    Pos = SearchQuoteByTime(this, Times[j], Pos);
//      Pos = SearchQuoteByTime(this, QuotesTimes[j].time, Pos);

    if (this[Pos].time < Times[j])
//      if (this[Pos].time < QuotesTimes[j].time)
    {
      //  ArrayCopy    count == 0       
      if (Pos >= PrevPos)
        NewAmount += ArrayCopy(TmpQuotes, this.Quotes, NewAmount, PrevPos, Pos - PrevPos + 1);

      PrevPos = Pos + 1;
      TmpQuotes[NewAmount].bid = this[Pos].bid.last;
      TmpQuotes[NewAmount].ask = this[Pos].ask.last;

      TmpQuotes[NewAmount].time = Times[j];
//        TmpQuotes[NewAmount].time = QuotesTimes[j].time;

      NewAmount++;
    }
  }

  NewAmount += ArrayCopy(TmpQuotes, this.Quotes, NewAmount, PrevPos); //  PrevPos == 0   (. )

  if (!this.MyArrayResize(this.Quotes, NewAmount, __FUNCTION__, FALSE))
    return;

  //    ,      
  ArrayCopy(this.Quotes, TmpQuotes);

  return;
}

//     [BeginTime; EndTime]
void QUOTES::FilterByInterval( const datetime BeginTime, datetime EndTime = NULL )
{
  if (this.GetAmount() <= 0)
    return;

  if (EndTime == NULL)
    EndTime = this.GetLastTime();

  int i = SearchQuoteByTime(this, BeginTime);
  const int EndPos = SearchQuoteByTime(this, EndTime, i);

  int j = 0;

//    ArrayCopy(this.Quotes, this.Quotes, 0, i, EndPos - i + 1); // : "     ,   "

  while (i <= EndPos)
  {
    this.Quotes[j] = this[i];

    j++;
    i++;
  }

  if (!this.MyArrayResize(this.Quotes, j, __FUNCTION__))
    return;

  return;
}

//        
static void QUOTES::Synchronize( QUOTES* &ArrayQuotes[] )
{
  const int Amount = ArraySize(ArrayQuotes);

  if (Amount <= 1)
    return;
/*
//   void AddTimePoints( const QUOTES* QuotesTimes ) - : O(n^2)
  for (int i = 0; i < Amount; i++)
    for (int j = 0; j < Amount; j++)
      if (i != j)
        if ((ArrayQuotes[i] != NULL))
          ArrayQuotes[i].AddTimePoints(ArrayQuotes[j]);
*/

//--------------------------------

//   void AddTimePoints( const datetime &Times[] ) - : O(n)
  int AmountTimes = 0;

  int Amounts[], Pos[];

  ArrayResize(Amounts, Amount);
  ArrayInitialize(Amounts, 0);

  ArrayResize(Pos, Amount);
  ArrayInitialize(Pos, 0);

  for (int i = 0; i < Amount; i++)
    if (ArrayQuotes[i] != NULL)
    {
      Amounts[i] = ArrayQuotes[i].GetAmount();

      AmountTimes += Amounts[i]; //  
    }

/*
  //     
  while (!IsStopped())
  {
    datetime MinTime = DATETIME_MAX;

    for (int i = 0; i < Amount; i++)
      if (Pos[i] < Amounts[i])
        if (ArrayQuotes[i][Pos[i]].time < MinTime)
          MinTime = ArrayQuotes[i][Pos[i]].time;

    if (MinTime == DATETIME_MAX)
      break;

    for (int i = 0; i < Amount; i++)
      if (Pos[i] < Amounts[i])
        if (ArrayQuotes[i][Pos[i]].time == MinTime)
          Pos[i]++;

//      Times[AmountTimes] = MinTime;
    AmountTimes++;
  }

  ArrayInitialize(Pos, 0);
*/
  datetime Times[];

  if (!QUOTES::MyArrayResize(Times, AmountTimes, __FUNCTION__))
    return;

  AmountTimes = 0;

  while (!IsStopped())
  {
    datetime MinTime = DATETIME_MAX;

    for (int i = 0; i < Amount; i++)
      if (Pos[i] < Amounts[i])
        if (ArrayQuotes[i][Pos[i]].time < MinTime)
          MinTime = ArrayQuotes[i][Pos[i]].time;

    if (MinTime == DATETIME_MAX)
      break;

    for (int i = 0; i < Amount; i++)
      if (Pos[i] < Amounts[i])
        if (ArrayQuotes[i][Pos[i]].time == MinTime)
          Pos[i]++;
    Times[AmountTimes] = MinTime;
    AmountTimes++;
  }

  if (!QUOTES::MyArrayResize(Times, AmountTimes, __FUNCTION__))
    return;

  for (int i = 0; i < Amount; i++)
    if (ArrayQuotes[i] != NULL)
      ArrayQuotes[i].AddTimePoints(Times); // void AddTimePoints( const datetime &Times[] )

//--------------------------------

  QUOTES::FilterByInterval(ArrayQuotes);

  return;
}