#property strict

#include <Quotes.mqh>

#define ALPHA 0.1

class QUOTES_FXOPEN : public QUOTES
{
private:
  string SymbolBid, SymbolAsk;

  //     
  static int PriceToInt( const double Price, const double point)
  {
    return((int)(Price / point + ALPHA));
  }

  void InitSymbols( void )
  {
    SymbolBid = this.GetSymbolName();
    SymbolAsk = SymbolBid + "_Ask";

    return;
  }

  //  
  bool GetPrices( const datetime BeginTime = NULL, datetime EndTime = NULL )
  {
    RefreshRates();

    MqlRates RatesBid[], RatesAsk[];

    if (!this.MyArrayCopyRates(RatesBid, this.SymbolBid, this.TimeFrame, __FUNCTION__))
      return(FALSE);

    if (!this.MyArrayCopyRates(RatesAsk, this.SymbolAsk, this.TimeFrame, __FUNCTION__))
      return(FALSE);

    ArraySetAsSeries(RatesBid, FALSE);
    ArraySetAsSeries(RatesAsk, FALSE);

    const int AmountBid = ArraySize(RatesBid);
    const int AmountAsk = ArraySize(RatesAsk);

    if (EndTime == NULL)
      EndTime = (RatesBid[AmountBid - 1].time > RatesAsk[AmountAsk - 1].time) ? RatesAsk[AmountAsk - 1].time : RatesBid[AmountBid - 1].time;

    int PosBid = QUOTES::SearchQuoteByTime(RatesBid, BeginTime);
    int PosAsk = QUOTES::SearchQuoteByTime(RatesAsk, BeginTime);

    if (RatesBid[PosBid].time < RatesAsk[PosAsk].time)
      PosBid = QUOTES::SearchQuoteByTime(RatesBid, RatesAsk[PosAsk].time, PosBid);
    else if (RatesBid[PosBid].time > RatesAsk[PosAsk].time)
      PosAsk = QUOTES::SearchQuoteByTime(RatesAsk, RatesBid[PosBid].time, PosAsk);

    const int PosBid0 = PosBid;
    const int PosAsk0 = PosAsk;

    int Amount = 0;

    while ((PosBid < AmountBid) && (PosAsk < AmountAsk) &&
           (RatesBid[PosBid].time <= EndTime) && (RatesAsk[PosAsk].time <= EndTime)) //    
    {
      const datetime TimeBid = RatesBid[PosBid].time;
      const datetime TimeAsk = RatesAsk[PosAsk].time;

      if (TimeBid < TimeAsk)
        PosBid++;
      else if (TimeBid > TimeAsk)
        PosAsk++;
      else // (TimeBid == TimeAsk)
      {
        PosBid++;
        PosAsk++;
      }

      Amount++;
    }

    //        (  )
    if (!this.MyArrayResize(this.Quotes, Amount, __FUNCTION__))
      return(FALSE);

    //     

    const double PointBid = SymbolInfoDouble(SymbolBid, SYMBOL_POINT);
    const double PointAsk = SymbolInfoDouble(SymbolAsk, SYMBOL_POINT);

    const int BarInterval = PeriodSeconds(this.TimeFrame);

    PosBid = PosBid0;
    PosAsk = PosAsk0;

    int i = 0;

    while ((i < Amount) && (PosBid < AmountBid) && (PosAsk < AmountAsk) &&
           (RatesBid[PosBid].time <= EndTime) && (RatesAsk[PosAsk].time <= EndTime)) //    
    {
      const MqlRates RateBid = RatesBid[PosBid];
      const MqlRates RateAsk = RatesAsk[PosAsk];

      if (RateBid.time < RateAsk.time) //  RateBid.time-
      {
        this.Quotes[i].bid.last = PriceToInt(RateBid.close, PointBid);
        this.Quotes[i].bid.min = PriceToInt(RateBid.low, PointBid);
        this.Quotes[i].bid.max = PriceToInt(RateBid.high, PointBid);

        //      (PosAsk > 0 -    )
        this.Quotes[i].ask = PriceToInt(RatesAsk[PosAsk - 1].close, PointAsk);

        //       : "  ". . https://www.mql5.com/ru/forum/5108#comment_146716
        //     -    .    .
        this.Quotes[i].time = RateBid.time + BarInterval; //    

        PosBid++;
      }
      else if (RateBid.time > RateAsk.time) //  RateAsk.time-
      {
        //      (PosBid > 0 -    )
        this.Quotes[i].bid = PriceToInt(RatesBid[PosBid - 1].close, PointBid);

        this.Quotes[i].ask.last = PriceToInt(RateAsk.close, PointAsk);
        this.Quotes[i].ask.min = PriceToInt(RateAsk.low, PointAsk);
        this.Quotes[i].ask.max = PriceToInt(RateAsk.high, PointAsk);

        //       : "  ". . https://www.mql5.com/ru/forum/5108#comment_146716
        //     -    .    .
        this.Quotes[i].time = RateAsk.time + BarInterval; //    

        PosAsk++;
      }
      else // (TimeBid == TimeAsk)
      {
        this.Quotes[i].bid.last = PriceToInt(RateBid.close, PointBid);
        this.Quotes[i].bid.min = PriceToInt(RateBid.low, PointBid);
        this.Quotes[i].bid.max = PriceToInt(RateBid.high, PointBid);

        this.Quotes[i].ask.last = PriceToInt(RateAsk.close, PointAsk);
        this.Quotes[i].ask.min = PriceToInt(RateAsk.low, PointAsk);
        this.Quotes[i].ask.max = PriceToInt(RateAsk.high, PointAsk);

        //       : "  ". . https://www.mql5.com/ru/forum/5108#comment_146716
        //     -    .    .
        this.Quotes[i].time = RateBid.time + BarInterval; //    

        PosBid++;
        PosAsk++;
      }

      i++;
    }

    return(TRUE);
  }
protected:
  template <typename T>
  static bool MyArrayCopyRates( T &Array[], const string Symb, const int TF, const string FuncName )
  {
    const bool Res = (ArrayCopyRates(Array, Symb, TF) != -1);

    if (!Res) //        ( )
      Alert(FuncName + ": "+ "ArrayCopyRates(" + typename(T) + ", " + Symb + ", " + (string)TF + ") - error " + (string)GetLastError());

    return(Res);
  }


public:
  QUOTES_FXOPEN( string sSymbolName = NULL, const datetime BeginTime = NULL, const datetime EndTime = NULL, const int iTimeFrame = NULL ) :
  QUOTES(StringSubstr(CheckSymbolName(sSymbolName), 0, 6), iTimeFrame)
  {
    InitSymbols();
    Init(BeginTime , EndTime, iTimeFrame);
  }

  QUOTES_FXOPEN( QUOTES &AnotherQuotes, const string sSymbolName = NULL ) :
  QUOTES(StringSubstr(CheckSymbolName(sSymbolName), 0, 6), AnotherQuotes.GetTimeFrame())
  {
    InitSymbols();
    Init(AnotherQuotes);
  }

  virtual bool Init( const datetime BeginTime = NULL, const datetime EndTime = NULL, const int iTimeFrame = NULL )
  {
    this.SetTimeFrame(iTimeFrame);

    return(GetPrices(BeginTime, EndTime));
  }

  virtual bool Init( const QUOTES &AnotherQuotes )
  {
    datetime BeginTime = NULL;
    datetime EndTime = NULL;

    this.SetTimeFrame(AnotherQuotes.GetTimeFrame());

    const int iAmount = AnotherQuotes.GetAmount();

    if (iAmount > 0)
    {
      BeginTime = AnotherQuotes[0].time - PeriodSeconds(this.GetTimeFrame());
      EndTime = AnotherQuotes[iAmount - 1].time - PeriodSeconds(this.GetTimeFrame());
    }

    return(GetPrices(BeginTime, EndTime));
  }
};